// DCC Decoder for DS-DCC decode
// By yaasan
// Based on Nicolas's sketch http://blog.nicolas.cx
// Inspired by Geoff Bunza and his 17 Function DCC Decoder & updated library
//
//
// Debug serial output available on the serial port at baud 115200, aka Tools -> Serial Monitor
/*****************************************************************************************
 * MP3 Sound Decoder for DS-DCC decode
 *
 *
 *
 *****************************************************************************************/

#include <string.h>
#include <NmraDcc.h>
//#include <avr/eeprom.h>	 //required by notifyCVRead() function if enabled below
#include "motor_func.h"
#include "motor_ctrl.h"
#include <SoftwareSerial.h>
#include <DFPlayer_Mini_Mp3.h>
//ソフトウエアシリアルの設定　
SoftwareSerial mySerial(3,4); // RX, TX
/*************************************************
 * 
 * Declaration of Symbols
 * 
 *************************************************/

//#define ATTINYUSE

#define DESIGNER_ID   255
#define PRODUCT_CODE  108

#define DECODER_ADDRESS 3
//#define DCC_ACK_PIN 9   //if defined enables the ACK pin functionality. Comment out to disable.

//CV
#define CV_VSTART 2               //起動電圧
#define CV_ACCRATIO 3             //加速率
#define CV_DECCRATIO  4           //減速率
#define CV_VMAX 5                 //最高電圧
#define CV_DECODER_MODE  30       //ファンクションモード
#define CV_BEMFcoefficient  54    //BEMF係数
#define CV_PI_P 55                //BEMF　PI_P
#define CV_PI_I 56                //BEMF  PI_I
#define CV_BEMFCUTOFF 57          //BEMF  ON/OFF  
#define CV_MP3_Vol 58             //MP3 Volume
#define CV_MP3_F0 110             //F00 Function
#define CV_MP3_F1 111             //F01 Function
#define CV_MP3_F2 112             //F02 Function
#define CV_MP3_F3 113             //F03 Function
#define CV_MP3_F4 114             //F04 Function
#define CV_MP3_F5 115             //F05 Function
#define CV_MP3_F6 116             //F06 Function
#define CV_MP3_F7 117             //F07 Function
#define CV_MP3_F8 118             //F08 Function
#define CV_MP3_F9 119             //F09 Function
#define CV_MP3_F10 120            //F10 Function
#define CV_MP3_F11 121            //F11 Function
#define CV_MP3_F12 122            //F12 Function
#define CV_MP3_F13 123            //F13 Function
#define CV_MP3_F14 124            //F14 Function
#define CV_MP3_F15 125            //F15 Function
#define CV_MP3_F16 126            //F16 Function
#define CV_MP3_F17 127            //F17 Function
#define CV_MP3_F18 128            //F18 Function
#define CV_MP3_F19 129            //F19 Function
#define CV_MP3_F20 130            //F20 Function
#define CV_MP3_F21 131            //F21 Function
#define CV_MP3_F22 132            //F22 Function
#define CV_MP3_F23 133            //F23 Function
#define CV_MP3_F24 134            //F24 Function
#define CV_MP3_F25 135            //F25 Function
#define CV_MP3_F26 136            //F26 Function
#define CV_MP3_F27 137            //F27 Function
#define CV_MP3_F28 138            //F28 Function

/*************************************************
 * 
 * Declaration of Classes
 * 
 *************************************************/

NmraDcc	 Dcc;
DCC_MSG	 Packet;

/*************************************************
 * 
 * Declaration of Variables
 * 
 *************************************************/

//Task Schedule
unsigned long gPreviousL5 = 0;
unsigned long gPreviousL4 = 0;

//モータ制御関連の変数
uint16_t gSpeedCmd = 0;
uint8_t gPwmDir = 128;
uint8_t gPwmDirv = 128;
uint8_t gCV29_Conf = 0;
uint8_t gCV29Direction = 0;
// CV STD
uint8_t gCV1_SAddr = 3;
uint8_t gCVx_LAddr = 3;
uint8_t gCV2_Vstart = 20;
uint8_t gCV3_AccRatio = 16;
uint8_t gCV4_DecRatio = 16;
uint8_t gCV5_VMAX = 255;
//MotorControl
uint8_t gCV30_DECODER_MODE = 0; // 0でモータ　2でFL両極性
uint8_t gCV54_BEMFcoefficient = 3; 
uint8_t gCV55_PI_P = 32;
uint8_t gCV56_PI_I = 96;
uint8_t gCV57_BEMF_CutOff = 0;  // 0でBEMF Off 1でBEMF On
//MP3 Control
uint8_t gCV58_MP3_Vol = 28;
uint8_t gCV110_MP3_F0 = 0;
uint8_t gCV111_MP3_F1 = 0;
uint8_t gCV112_MP3_F2 = 1;
uint8_t gCV113_MP3_F3 = 1;
uint8_t gCV114_MP3_F4 = 1;
uint8_t gCV115_MP3_F5 = 1;
uint8_t gCV116_MP3_F6 = 1;
uint8_t gCV117_MP3_F7 = 1;
uint8_t gCV118_MP3_F8 = 1;
uint8_t gCV119_MP3_F9 = 1;
uint8_t gCV120_MP3_F10 = 1;
uint8_t gCV121_MP3_F11 = 1;
uint8_t gCV122_MP3_F12 = 1;
uint8_t gCV123_MP3_F13 = 1;
uint8_t gCV124_MP3_F14 = 1;
uint8_t gCV125_MP3_F15 = 1;
uint8_t gCV126_MP3_F16 = 1;
uint8_t gCV127_MP3_F17 = 1;
uint8_t gCV128_MP3_F18 = 1;
uint8_t gCV129_MP3_F19 = 1;
uint8_t gCV130_MP3_F20 = 1;
uint8_t gCV131_MP3_F21 = 1;
uint8_t gCV132_MP3_F22 = 1;
uint8_t gCV133_MP3_F23 = 1;
uint8_t gCV134_MP3_F24 = 1;
uint8_t gCV135_MP3_F25 = 1;
uint8_t gCV136_MP3_F26 = 1;
uint8_t gCV137_MP3_F27 = 1;
uint8_t gCV138_MP3_F28 = 1;

//ファンクションの変数
uint8_t fn_bit_f0 = 0;
uint8_t fn_bit_f1 = 0;
uint8_t fn_bit_f2 = 0;
uint8_t fn_bit_f3 = 0;
uint8_t fn_bit_f4 = 0;
uint8_t fn_bit_f5 = 0;
uint8_t fn_bit_f6 = 0;
uint8_t fn_bit_f7 = 0;
uint8_t fn_bit_f8 = 0;
uint8_t fn_bit_f9 = 0;
uint8_t fn_bit_f10 = 0;
uint8_t fn_bit_f11 = 0;
uint8_t fn_bit_f12 = 0;
uint8_t fn_bit_f13 = 0;
uint8_t fn_bit_f14 = 0;
uint8_t fn_bit_f15 = 0;
uint8_t fn_bit_f16 = 0;
uint8_t fn_bit_f17 = 0;
uint8_t fn_bit_f18 = 0;
uint8_t fn_bit_f19 = 0;
uint8_t fn_bit_f20 = 0;
uint8_t fn_bit_f21 = 0;
uint8_t fn_bit_f22 = 0;
uint8_t fn_bit_f23 = 0;
uint8_t fn_bit_f24 = 0;
uint8_t fn_bit_f25 = 0;
uint8_t fn_bit_f26 = 0;
uint8_t fn_bit_f27 = 0;
uint8_t fn_bit_f28 = 0;
uint8_t fn_f0_sw = 0;



//Internal variables and other.
#if defined(DCC_ACK_PIN)
const int DccAckPin = DCC_ACK_PIN ;
#endif

struct CVPair{
  uint16_t	CV;
  uint8_t	Value;
};
CVPair FactoryDefaultCVs [] = {
  {CV_MULTIFUNCTION_PRIMARY_ADDRESS, DECODER_ADDRESS},
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},    //The LSB is set CV 1 in the libraries .h file, which is the regular address location, so by setting the MSB to 0 we tell the library to use the same address as the primary address. 0 DECODER_ADDRESS
  {CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB, 0},  //XX in the XXYY address
  {CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB, 0},  //YY in the XXYY address
  {CV_29_CONFIG, 2},   //Make sure this is 0 or else it will be random based on what is in the eeprom which could caue headaches
  {CV_VSTART, 20},
  {CV_ACCRATIO, 16},
  {CV_DECCRATIO, 16},
  {CV_VMAX, 255},
  {CV_DECODER_MODE, 0},
  {CV_BEMFcoefficient, 3},
  {CV_PI_P, 32},
  {CV_PI_I, 96},
  {CV_BEMFCUTOFF,0},
  {CV_MP3_Vol,28},
  {CV_MP3_F0,0},
  {CV_MP3_F1,0},
  {CV_MP3_F2,1},
  {CV_MP3_F3,1},
  {CV_MP3_F4,1},
  {CV_MP3_F5,1},
  {CV_MP3_F6,1},
  {CV_MP3_F7,1},
  {CV_MP3_F8,1},
  {CV_MP3_F9,1},
  {CV_MP3_F10,1},
  {CV_MP3_F11,1},
  {CV_MP3_F12,1},
  {CV_MP3_F13,1},
  {CV_MP3_F14,1},
  {CV_MP3_F15,1},
  {CV_MP3_F16,1},
  {CV_MP3_F17,1},
  {CV_MP3_F18,1},
  {CV_MP3_F19,1},
  {CV_MP3_F20,1},
  {CV_MP3_F21,1},
  {CV_MP3_F22,1},
  {CV_MP3_F23,1},
  {CV_MP3_F24,1},
  {CV_MP3_F25,1},
  {CV_MP3_F26,1},
  {CV_MP3_F27,1},
  {CV_MP3_F28,1}
};

/*************************************************
 * 
 * Declaration of Functions
 * 
 *************************************************/

void(* resetFunc) (void) = 0;  //declare reset function at address 0

uint8_t FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);
uint16_t limitSpeed(uint16_t inSpeed);
void loop_config();
void CVCFG_ShowAllCV();
String CVCFG_ReceiveRequest();
word CVCFG_StrToWord(String s);

/*************************************************
 * 
 * Functions
 * 
 *************************************************/

void notifyCVResetFactoryDefault()
{
  //When anything is writen to CV8 reset to defaults. 

  resetCVToDefault();  
  Serial.println("Resetting...");
  delay(1000);  //typical CV programming sends the same command multiple times - specially since we dont ACK. so ignore them by delaying
  resetFunc();
}

void resetCVToDefault()
{
  //CVをデフォルトにリセット
  Serial.println("CVs being reset to factory defaults");
  
  for (int j=0; j < FactoryDefaultCVIndex; j++ ){
    Dcc.setCV( FactoryDefaultCVs[j].CV, FactoryDefaultCVs[j].Value);
  }
}

extern void notifyCVChange( uint16_t CV, uint8_t Value)
{
   //CVが変更されたときのメッセージ
   Serial.print("CV "); 
   Serial.print(CV); 
   Serial.print(" Changed to "); 
   Serial.println(Value, DEC);
}

void notifyCVAck(void)
{
  Serial.println("notifyCVAck");
  MOTOR_Ack();
}

void setup()
{
	uint8_t cv_value;
	
	//シリアル通信開始
	mySerial.begin (9600);
  Serial.begin (115200);
  
	//MP3 Player configuration
	mp3_set_serial (mySerial);	//set softwareSerial for DFPlayer-mini mp3 module 
    
	//D9,D10 PWM キャリア周期:31kHz
	TCCR1B &= B11111000;
	TCCR1B |= B00000001;
	
	//PWM出力ピン D9,D10を出力にセット
	pinMode(9, OUTPUT);
	pinMode(10, OUTPUT);
  //設定モード用
  pinMode(13, INPUT);// SCK pin sets input 
  digitalWrite(13, HIGH); //Internal pull up enabled

  
  #if !defined(DECODER_DONT_DEFAULT_CV_ON_POWERUP)
   if ( Dcc.getCV(CV_MULTIFUNCTION_PRIMARY_ADDRESS) == 0xFF ){  //if eeprom has 0xFF then assume it needs to be programmed
      Serial.println("CV Defaulting due to blank eeprom");
      notifyCVResetFactoryDefault();
   }
   else{
      Serial.println("CV Not Defaulting");
   }
  #else
    Serial.println("CV Defaulting Always On Powerup");
    notifyCVResetFactoryDefault();
  #endif 

  // Setup which External Interrupt, the Pin it's associated with that we're using, disable pullup.
  Dcc.pin(0, 2, 0);

  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init( MAN_ID_DIY, 100,   FLAGS_MY_ADDRESS_ONLY , 0 ); 

  //Reset task
  gPreviousL5 = millis();

  //ＣＶ値読み出し
  gCV1_SAddr = Dcc.getCV( CV_MULTIFUNCTION_PRIMARY_ADDRESS ) ;
  gCV2_Vstart = Dcc.getCV( CV_VSTART ) ;
  gCV3_AccRatio = Dcc.getCV( CV_ACCRATIO ) ;
  gCV4_DecRatio = Dcc.getCV( CV_DECCRATIO ) ;
  gCV5_VMAX = Dcc.getCV( CV_VMAX ) ;
  gCVx_LAddr = (Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB ) << 8) + Dcc.getCV( CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB );
  gCV29_Conf = Dcc.getCV( CV_29_CONFIG ) ;
  gCV30_DECODER_MODE = Dcc.getCV( CV_DECODER_MODE ) ;
  gCV54_BEMFcoefficient =  Dcc.getCV( CV_BEMFcoefficient ) ;
  gCV55_PI_P = Dcc.getCV( CV_PI_P ) ;
  gCV56_PI_I = Dcc.getCV( CV_PI_I ) ;
  gCV57_BEMF_CutOff = Dcc.getCV( CV_BEMFCUTOFF ) ;
  gCV58_MP3_Vol = Dcc.getCV( CV_MP3_Vol ) ;
  gCV110_MP3_F0 = Dcc.getCV( CV_MP3_F0 ) ;
  gCV111_MP3_F1 = Dcc.getCV( CV_MP3_F1 ) ;
  gCV112_MP3_F2 = Dcc.getCV( CV_MP3_F2 ) ;
  gCV113_MP3_F3 = Dcc.getCV( CV_MP3_F3 ) ;
  gCV114_MP3_F4 = Dcc.getCV( CV_MP3_F4 ) ;
  gCV115_MP3_F5 = Dcc.getCV( CV_MP3_F5 ) ;
  gCV116_MP3_F6 = Dcc.getCV( CV_MP3_F6 ) ;
  gCV117_MP3_F7 = Dcc.getCV( CV_MP3_F7 ) ;
  gCV118_MP3_F8 = Dcc.getCV( CV_MP3_F8 ) ;
  gCV119_MP3_F9 = Dcc.getCV( CV_MP3_F9 ) ;
  gCV120_MP3_F10 = Dcc.getCV( CV_MP3_F10 ) ;
  gCV121_MP3_F11 = Dcc.getCV( CV_MP3_F11 ) ;
  gCV122_MP3_F12 = Dcc.getCV( CV_MP3_F12 ) ;
  gCV123_MP3_F13 = Dcc.getCV( CV_MP3_F13 ) ;
  gCV124_MP3_F14 = Dcc.getCV( CV_MP3_F14 ) ;
  gCV125_MP3_F15 = Dcc.getCV( CV_MP3_F15 ) ;
  gCV126_MP3_F16 = Dcc.getCV( CV_MP3_F16 ) ;
  gCV127_MP3_F17 = Dcc.getCV( CV_MP3_F17 ) ;
  gCV128_MP3_F18 = Dcc.getCV( CV_MP3_F18 ) ;
  gCV129_MP3_F19 = Dcc.getCV( CV_MP3_F19 ) ;
  gCV130_MP3_F20 = Dcc.getCV( CV_MP3_F20 ) ;
  gCV131_MP3_F21 = Dcc.getCV( CV_MP3_F21 ) ;
  gCV132_MP3_F22 = Dcc.getCV( CV_MP3_F22 ) ;
  gCV133_MP3_F23 = Dcc.getCV( CV_MP3_F23 ) ;
  gCV134_MP3_F24 = Dcc.getCV( CV_MP3_F24 ) ;
  gCV135_MP3_F25 = Dcc.getCV( CV_MP3_F25 ) ;
  gCV136_MP3_F26 = Dcc.getCV( CV_MP3_F26 ) ;
  gCV137_MP3_F27 = Dcc.getCV( CV_MP3_F27 ) ;
  gCV138_MP3_F28 = Dcc.getCV( CV_MP3_F28 ) ;
  

//cv29 方転チェック
  if (gCV29_Conf & 1)
  {
      gCV29Direction =1;
  }
  else{
      gCV29Direction =0;    
  }

//モータ制御の初期設定
	MOTOR_Init();
	MOTOR_SetCV(2, gCV2_Vstart);
  MOTOR_SetCV(54, gCV54_BEMFcoefficient);        // Nagoden Adjust1 2016/06/02
  MOTOR_SetCV(55, gCV55_PI_P);                   // MECY Adjust1 2016/04/09
  MOTOR_SetCV(56, gCV56_PI_I);                   // MECY Adjust1 2016/04/09
  MOTOR_SetCV(57, gCV57_BEMF_CutOff);


/*
//Debug message  
  Serial.print("CV1(ShortAddr): ");
  Serial.println(gCV1_SAddr);
  Serial.print("CV17/18(LongAddr): ");
  Serial.println(gCVx_LAddr);
  Serial.print("CV2(Vstart): ");
  Serial.println(gCV2_Vstart);
  Serial.print("CV3(AccRatio): ");
  Serial.println(gCV3_AccRatio);
  Serial.print("CV4(DecRatio): ");
  Serial.println(gCV4_DecRatio);
  Serial.print("CV5(VoltageMAX): ");
  Serial.println(gCV5_VMAX);
  Serial.print("CV29(CV29_Config): ");
  Serial.println(gCV29_Conf);
  Serial.print("CV30(DecoderMode): ");
  Serial.println(gCV30_DECODER_MODE);
  Serial.print("CV54 (BEMF coefficient): ");
  Serial.println(gCV54_BEMFcoefficient);
  Serial.print("CV55 (PI P Gain): ");
  Serial.println(gCV55_PI_P);
  Serial.print("CV56 (PI I gain): ");
  Serial.println(gCV56_PI_I);
  Serial.print("CV57 (BEMF Cut Off): ");
  Serial.println(gCV57_BEMF_CutOff);
  Serial.print("CV58 (MP3 Volume): ");
  Serial.println(gCV58_MP3_Vol);
  Serial.println("Ready");
*/

}

void loop()
{

	// You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
	Dcc.process();

	/* DCC motor control */
	if( (millis() - gPreviousL4) >= 10)
	{
   //ＣＶ３０が０の時モータ制御をする
    if (gCV30_DECODER_MODE == 0)  //デコーダモードが　0：モータ時
    {
		  //Speed detection
  		  MOTOR_Sensor();
			//Reset task
		  gPreviousL4 = millis();
    } 
	}	
	
	if( (millis() - gPreviousL5) >= 100)
	{
   //ＣＶ３０が０の時モータ制御をする 
    if (gCV30_DECODER_MODE == 0)  //デコーダモードが　0：モータ時
    {
      	//Motor drive control
	  	    MOTOR_Main(gSpeedCmd, gPwmDirv);
      	//Reset task
		    gPreviousL5 = millis();
    }   
	}
}


//DCC速度信号の受信によるイベント
extern void notifyDccSpeed( uint16_t Addr, uint8_t Speed, uint8_t ForwardDir, uint8_t MaxSpeed )
{
	uint16_t aSpeedRef = 0;
	
	//速度値の正規化(255を100%とする処理)
	if( Speed >= 1)
	{
		aSpeedRef = ((Speed - 1) * 255) / MaxSpeed;
	}
	else
	{
    //緊急停止信号受信時の処理 //Nagoden comment 2016/06/11
    Serial.println("***** Emagency STOP **** ");
    aSpeedRef = 0;
  }
	
	//リミッタ
	if(aSpeedRef > gCV5_VMAX)
	{
		aSpeedRef = gCV5_VMAX;
	}
	
	gSpeedCmd = aSpeedRef;
	gPwmDir = ForwardDir;
	if ( gCV29Direction > 0)
  {
     if ( ForwardDir == 0)
     {
          gPwmDirv =0;
     }
    else
    {
          gPwmDirv =128;     
    }     
  }
  else
  {
     if ( ForwardDir == 0)
     {
          gPwmDirv =128;
     }
    else
    {
         gPwmDirv =0;     
    }     
  }

  if (gCV30_DECODER_MODE == 2)
  {
   exec_function (fn_f0_sw);
  }

	// デバッグメッセージ
	//Serial.print("Speed - ADR: ");
	//Serial.print(Addr);
	//Serial.print(", SPD: ");
	//Serial.print(Speed);
	//Serial.print(", DIR: ");
	//Serial.print(ForwardDir);
	//Serial.print(", MAX: ");
	//Serial.println(MaxSpeed);

}

//ファンクション信号受信のイベント

extern void notifyDccFunc( uint16_t Addr, FN_GROUP FuncGrp, uint8_t FuncState)
{
	switch(FuncGrp)
	{
    case FN_0_4:
//      Function F00
         if  ((FuncState & FN_BIT_00) == 16)
         {
            if (fn_bit_f0 == 0 && gCV110_MP3_F0 != 0)
            {
              fn_bit_f0 = 1;
              if (gCV30_DECODER_MODE == 2)  //デコーダモードが　2：ヘッドライトの場合
              {
                Serial.println("fn_f0_sw = 1");
                fn_f0_sw = 1 ;
              }
              else
              {
                mp3_set_volume (gCV58_MP3_Vol);
                delay (10);
                mp3_play (0) ;
                if (gCV110_MP3_F0 == 2)
                {
                  delay (10);
                  mp3_single_loop (true); 
                }
              }  
            }
         }    
         else
         {
            if (fn_bit_f0 == 1 && gCV110_MP3_F0 != 0) 
            {
              fn_bit_f0 = 0;
              if (gCV30_DECODER_MODE == 2)  //デコーダモードが　2：ヘッドライトの場合
              {
                Serial.println("fn_f0_sw = 0");
                fn_f0_sw = 0;
              }
              else
              {
                if (gCV110_MP3_F0 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV110_MP3_F0 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (0) ;
                }
              }  
            }
         }
             
//      Function F01
         if  ((FuncState & FN_BIT_01) == 1){
            if (fn_bit_f1 == 0 && gCV111_MP3_F1 != 0) 
            {
              fn_bit_f1 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (1) ;
              if (gCV111_MP3_F1 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }   
            }
         }    
         else
         {
            if (fn_bit_f1 == 1 && gCV111_MP3_F1 != 0)
            {
                fn_bit_f1 = 0;
                if (gCV111_MP3_F1 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV111_MP3_F1 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (1) ;
                }
            }
         }
//      Function F02
         if  ((FuncState & FN_BIT_02) == 2) {
            if (fn_bit_f2 == 0 && gCV112_MP3_F2 != 0) 
            {
              fn_bit_f2 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (2) ;
              if (gCV112_MP3_F2 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else {
            if (fn_bit_f2 == 1 && gCV112_MP3_F2 != 0) {
              fn_bit_f2 = 0;
              if (gCV112_MP3_F2 == 2)
              {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
              }    
              if (gCV112_MP3_F2 == 1)
              {
                  //mp3_set_volume (gCV58_MP3_Vol);
                  //delay (10);
                  //mp3_play (2) ;
              }
            }
         }
//      Function F03
         if  ((FuncState & FN_BIT_03) == 4)  
         {
            if (fn_bit_f3 == 0 && gCV113_MP3_F3 != 0)
            {
              fn_bit_f3 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (3) ;
              if (gCV113_MP3_F3 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else
         {
            if (fn_bit_f3 == 1 && gCV113_MP3_F3 != 0) 
            {
              fn_bit_f3 = 0;
              if (gCV113_MP3_F3 == 2)
              {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
              }    
              if (gCV113_MP3_F3 == 1)
              {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (3) ;
              }
            }
         }
//      Function F04
         if ((FuncState & FN_BIT_04) == 8)
         {
            if (fn_bit_f4 == 0 && gCV114_MP3_F4 != 0)
            {
              fn_bit_f4 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (4) ;
              if (gCV114_MP3_F4 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else
         {
            if (fn_bit_f4 == 1 && gCV114_MP3_F4 != 0) 
            {
              fn_bit_f4 = 0;
              if (gCV114_MP3_F4 == 2)
              {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
              }    
              if (gCV114_MP3_F4 == 1)
              {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (4) ;
              }
            }
         }
      break;
      
    case FN_5_8:
//      Function F05    
         if  ((FuncState & FN_BIT_05) == 1)
         {
            if (fn_bit_f5 == 0 && gCV115_MP3_F5 != 0) 
            {
              fn_bit_f5 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (5) ;
              if (gCV115_MP3_F5 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }

            }
         }    
         else
         {
            if (fn_bit_f5 == 1 && gCV115_MP3_F5 != 0)
            {
                fn_bit_f5 = 0;
                if (gCV115_MP3_F5 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV115_MP3_F5 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (5) ;
                }
            }
         }
//      Function F06    
         if  ((FuncState & FN_BIT_06) == 2) 
         {
            if (fn_bit_f6 == 0 && gCV116_MP3_F6 != 0) 
            {
              fn_bit_f6 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (6) ;
              if (gCV116_MP3_F6 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f6 == 1 && gCV116_MP3_F6 != 0) 
            {
                fn_bit_f6 = 0;
                if (gCV116_MP3_F6 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV116_MP3_F6 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (6) ;
                }
            }
         }
//      Function F07    
         if  ((FuncState & FN_BIT_07) == 4) 
         {
            if (fn_bit_f7 == 0 && gCV117_MP3_F7 != 0) 
            {
              fn_bit_f7 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (7);
              if (gCV117_MP3_F7 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f7 == 1 && gCV117_MP3_F7 != 0) 
            {
                fn_bit_f7 = 0;
                if (gCV117_MP3_F7 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV117_MP3_F7 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (7) ;
                }
            }
         }
//      Function F08    
         if  ((FuncState & FN_BIT_08) == 8)
         {
            if (fn_bit_f8 == 0 && gCV118_MP3_F8 != 0)
            {
              fn_bit_f8 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (8) ;
              if (gCV118_MP3_F8 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else
         {
            if (fn_bit_f8 == 1 && gCV118_MP3_F8 != 0)
            {
                fn_bit_f8 = 0;
                if (gCV118_MP3_F8 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV118_MP3_F8 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (8) ;
                }
            }
         }

      break;
    case FN_9_12:
//      Function F09    
         if  ((FuncState & FN_BIT_09) == 1) 
         {
            if (fn_bit_f9 == 0 && gCV119_MP3_F9 != 0) 
            {
              fn_bit_f9 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (9) ;
              if (gCV119_MP3_F9 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else
         {
            if (fn_bit_f9 == 1 && gCV119_MP3_F9 != 0) 
            {
                fn_bit_f9 = 0;
                if (gCV119_MP3_F9 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV119_MP3_F9 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (9) ;
                }
            }
         }
//      Function F10    
         if  ((FuncState & FN_BIT_10) == 2) 
         {
            if (fn_bit_f10 == 0 && gCV120_MP3_F10 != 0) 
            {
              fn_bit_f10 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (10) ;
              if (gCV120_MP3_F10 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f10 == 1 && gCV120_MP3_F10 != 0) 
            {
                fn_bit_f10 = 0;
                if (gCV120_MP3_F10 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV120_MP3_F10 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (10) ;
                }
            }
         }
//      Function F11    
         if  ((FuncState & FN_BIT_11) == 4) 
         {
            if (fn_bit_f11 == 0 && gCV121_MP3_F11 != 0) 
            {
              fn_bit_f11 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (11) ;
              if (gCV121_MP3_F11 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f11 == 1 && gCV121_MP3_F11 != 0) 
            {
                fn_bit_f11 = 0;
                if (gCV121_MP3_F11 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV121_MP3_F11 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (11) ;
                }
            }
         }

//      Function F12    
         if  ((FuncState & FN_BIT_12) == 8) 
         {
            if (fn_bit_f12 == 0 && gCV122_MP3_F12 != 0) 
            {
              fn_bit_f12 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (12) ;
              if (gCV122_MP3_F12 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f12 == 1 && gCV122_MP3_F12 != 0) 
            {
                fn_bit_f12 = 0;
                if (gCV122_MP3_F12 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV122_MP3_F12 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (12) ;
                }
            }
         }

      break;

    case FN_13_20:
//      Function F13
         if  ((FuncState & FN_BIT_13) == 1) 
         {
            if (fn_bit_f13 == 0 && gCV123_MP3_F13 != 0) 
            {
              fn_bit_f13 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (13) ;
              if (gCV123_MP3_F13 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f13 == 1 && gCV123_MP3_F13 != 0) 
            {
                fn_bit_f13 = 0;
                if (gCV123_MP3_F13 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV123_MP3_F13 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (13) ;
                }
            }
         }
//      Function F14    
         if  ((FuncState & FN_BIT_14) == 2) 
         {
            if (fn_bit_f14 == 0 && gCV124_MP3_F14 != 0) 
            {
              fn_bit_f14 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (14) ;
              if (gCV124_MP3_F14 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else {
            if (fn_bit_f14 == 1 && gCV124_MP3_F14 != 0) 
            {
                fn_bit_f14 = 0;
                if (gCV124_MP3_F14 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV124_MP3_F14 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (14) ;
                }
            }
         }
//      Function F15    
         if  ((FuncState & FN_BIT_15) == 4) 
         {
            if (fn_bit_f15 == 0 && gCV125_MP3_F15 != 0) 
            {
              fn_bit_f15 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (15) ;
              if (gCV125_MP3_F15 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f15 == 1 && gCV125_MP3_F15 != 0) 
            {
                fn_bit_f15 = 0;
                if (gCV125_MP3_F15 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV125_MP3_F15 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (15) ;
                }

            }
         }

//      Function F16    
         if  ((FuncState & FN_BIT_16) == 8) 
         {
            if (fn_bit_f16 == 0 && gCV126_MP3_F16 != 0) 
            {
              fn_bit_f16 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (16) ;
              if (gCV126_MP3_F16 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f16 == 1 && gCV126_MP3_F16 != 0) 
            {
               fn_bit_f16 = 0;
                if (gCV126_MP3_F16 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV126_MP3_F16 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (16) ;
                }
            }
         }
//      Function F17    
         if  ((FuncState & FN_BIT_17) == 16) 
         {
            if (fn_bit_f17 == 0 && gCV127_MP3_F17 != 0) 
            {
              fn_bit_f17 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (17) ;
              if (gCV127_MP3_F17 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f17 == 1 && gCV127_MP3_F17 != 0) 
            {
                fn_bit_f17 = 0;
                if (gCV127_MP3_F17 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV127_MP3_F17 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (17) ;
                }
            }
         }
//      Function F18    
         if  ((FuncState & FN_BIT_18) == 32) 
         {
            if (fn_bit_f18 == 0 && gCV128_MP3_F18 != 0) 
            {
              fn_bit_f18 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (18) ;
              if (gCV128_MP3_F18 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f18 == 1 && gCV128_MP3_F18 != 0) 
            {
                fn_bit_f18 = 0;
                if (gCV128_MP3_F18 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV128_MP3_F18 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (18) ;
                }
            }
         }
//      Function F19    
         if  ((FuncState & FN_BIT_19) == 64) 
         {
            if (fn_bit_f19 == 0 && gCV129_MP3_F19 != 0) 
            {
              fn_bit_f19 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (19) ;
              if (gCV129_MP3_F19 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f19 == 1 && gCV129_MP3_F19 != 0) 
            {
                fn_bit_f19 = 0;
                if (gCV129_MP3_F19 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV129_MP3_F19 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (19) ;
                }
            }
         }

 //      Function F20    
        if  ((FuncState & FN_BIT_20) == 128) 
        {
            if (fn_bit_f20 == 0 && gCV130_MP3_F20 != 0) 
            {
              fn_bit_f20 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (20) ;
              if (gCV130_MP3_F20 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f20 == 1 && gCV130_MP3_F20 != 0)
            {
                fn_bit_f20 = 0;
                if (gCV130_MP3_F20 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV130_MP3_F20 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (20) ;
                }
            }
         }
      break;
      
    case FN_21_28:
//      Function F21    

         if  ((FuncState & FN_BIT_21) == 1) 
         {
            if (fn_bit_f21 == 0 && gCV131_MP3_F21 != 0) 
            {
              fn_bit_f21 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (21) ;
              if (gCV131_MP3_F21 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f21 == 1 && gCV131_MP3_F21 != 0) 
            {
                fn_bit_f21 = 0;
                if (gCV131_MP3_F21 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV131_MP3_F21 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (21) ;
                }

             }
         }
//      Function F22    
         if  ((FuncState & FN_BIT_22) == 2) 
         {
            if (fn_bit_f22 == 0 && gCV132_MP3_F22 != 0) 
            {
              fn_bit_f22 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (22) ;
              if (gCV132_MP3_F22 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f22 == 1 && gCV132_MP3_F22 != 0) 
            {
                fn_bit_f22 = 0;
                if (gCV132_MP3_F22 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV132_MP3_F22 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (22) ;
                }

            }
         }
//      Function F23    
         if  ((FuncState & FN_BIT_23) == 4) 
         {
            if (fn_bit_f23 == 0 && gCV133_MP3_F23 != 0) 
            {
              fn_bit_f23 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (23) ;
              if (gCV133_MP3_F23 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f23 == 1 && gCV133_MP3_F23 != 0) 
            {
                fn_bit_f23 = 0;
                if (gCV133_MP3_F23 == 2)
                {
                    mp3_single_loop (false) ;
                    delay (10);
                    mp3_stop () ;
                }    
                if (gCV133_MP3_F23 == 1)
                {
                    mp3_set_volume (gCV58_MP3_Vol);
                    delay (10);
                    mp3_play (23) ;
                }
            }
         }
//      Function F24    
         if  ((FuncState & FN_BIT_24) == 8) 
         {
            if (fn_bit_f24 == 0 && gCV134_MP3_F24 != 0) 
            {
              fn_bit_f24 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (24) ;
              if (gCV134_MP3_F24 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f24 == 1 && gCV134_MP3_F24 != 0) 
            {
                fn_bit_f24 = 0;
                if (gCV134_MP3_F24 == 2)
                {
                    mp3_single_loop (false) ;
                    delay (10);
                    mp3_stop () ;
                }    
                if (gCV134_MP3_F24 == 1)
                {
                    mp3_set_volume (gCV58_MP3_Vol);
                    delay (10);
                    mp3_play (24) ;
                }
            }
         }
//      Function F25    
         if  ((FuncState & FN_BIT_25) == 16) 
         {
            if (fn_bit_f25 == 0 && gCV135_MP3_F25 != 0) 
            {
              fn_bit_f25 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (25) ;
              if (gCV135_MP3_F25 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f25 == 1 && gCV135_MP3_F25 != 0) 
            {
                fn_bit_f25 = 0;
                if (gCV135_MP3_F25 == 2)
                {
                  mp3_single_loop (false) ;
                  delay (10);
                  mp3_stop () ;
                }    
                if (gCV135_MP3_F25 == 1)
                {
                  mp3_set_volume (gCV58_MP3_Vol);
                  delay (10);
                  mp3_play (25) ;
                }
            }
         }
//      Function F26    
         if  ((FuncState & FN_BIT_26) == 32) 
         {
            if (fn_bit_f26 == 0 && gCV136_MP3_F26 != 0) 
            {
              fn_bit_f26 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (26) ;
              if (gCV136_MP3_F26 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f26 == 1 && gCV136_MP3_F26 != 0) 
            {
                fn_bit_f26 = 0;
                if (gCV136_MP3_F26 == 2)
                {
                    mp3_single_loop (false) ;
                    delay (10);
                    mp3_stop () ;
                }    
                if (gCV136_MP3_F26 == 1)
                {
                    mp3_set_volume (gCV58_MP3_Vol);
                    delay (10);
                    mp3_play (26) ;
                }
            }
         }
//      Function F27    
         if  ((FuncState & FN_BIT_27) == 64) 
         {
            if (fn_bit_f27 == 0 && gCV137_MP3_F27 != 0) 
            {
              fn_bit_f27 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (27) ;
              if (gCV137_MP3_F27 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else 
         {
            if (fn_bit_f27 == 1 && gCV137_MP3_F27 != 0) 
            {
                fn_bit_f27 = 0;
                if (gCV137_MP3_F27 == 2)
                {
                    mp3_single_loop (false) ;
                    delay (10);
                    mp3_stop () ;
                }    
                if (gCV137_MP3_F27 == 1)
                {
                   mp3_set_volume (gCV58_MP3_Vol);
                   delay (10);
                   mp3_play (27) ;
                }
            }
         }
 //      Function F28    
         if  ((FuncState & FN_BIT_28) == 128) 
         {
            if (fn_bit_f28 == 0 && gCV138_MP3_F28 != 0) 
            {
              fn_bit_f28 = 1;
              mp3_set_volume (gCV58_MP3_Vol);
              delay (10);
              mp3_play (28) ;
              if (gCV138_MP3_F28 == 2)
              {
                  delay (10);
                  mp3_single_loop (true); 
              }
            }
         }    
         else
         {
            if (fn_bit_f28 == 1 && gCV138_MP3_F28 != 0)
            {
                fn_bit_f28 = 0;
                if (gCV138_MP3_F28 == 2)
                {
                    mp3_single_loop (false) ;
                    delay (10);
                    mp3_stop () ;
                }    
                if (gCV138_MP3_F28 == 1)
               {
                    mp3_set_volume (gCV58_MP3_Vol);
                    delay (10);
                    mp3_play (28) ;
               }
            }
         }
     break;
	}
}
void exec_function (int fn_f0_sw)
{
    if (fn_f0_sw == 0)
    {
      analogWrite(MOTOR_PWM_A, 0);
      analogWrite(MOTOR_PWM_B, 0);
    }
    else
    {
      if (gPwmDirv ==0)
      {
      analogWrite(MOTOR_PWM_A, 255);
      analogWrite(MOTOR_PWM_B, 0);
      }
      else
      {
      analogWrite(MOTOR_PWM_A, 0);
      analogWrite(MOTOR_PWM_B, 255);
      }
    }
}



